@@ -12,13 +12,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from collections import deque
 import copy
 import dataclasses
 from functools import reduce
 import itertools
 from lxml import etree  # pytype: disable=import-error
 import re
-from typing import List, Optional, Sequence, Tuple
+from typing import Generator, Iterable, List, Optional, Sequence, Tuple
 from picosvg.svg_meta import (
     number_or_percentage,
     ntos,
@@ -27,6 +28,7 @@ from picosvg.svg_meta import (
     svgns,
     xlinkns,
     parse_css_declarations,
+    parse_view_box,
 )
 from picosvg.svg_types import *
 from picosvg.svg_transform import Affine2D
@@ -193,20 +195,16 @@ class SVG:
         self.elements[idx] = (el, shapes)
 
     def view_box(self) -> Optional[Rect]:
-        raw_box = self.svg_root.attrib.get("viewBox", None)
-        if not raw_box:
+        if "viewBox" not in self.svg_root.attrib:
             # if there is no explicit viewbox try to use width/height
             w = self.svg_root.attrib.get("width", None)
             h = self.svg_root.attrib.get("height", None)
             if w and h:
                 return Rect(0, 0, float(w), float(h))
+            else:
+                return None
 
-        if not raw_box:
-            return None
-        box = tuple(float(v) for v in re.split(r",|\s+", raw_box))
-        if len(box) != 4:
-            raise ValueError("Unable to parse viewBox")
-        return Rect(*box)
+        return parse_view_box(self.svg_root.attrib["viewBox"])
 
     def _default_tolerance(self):
         vbox = self.view_box()
@@ -845,7 +843,7 @@ class SVG:
 
         self._update_etree()
 
-        for tag in ("title", "desc", "metadata"):
+        for tag in ("title", "desc", "metadata", "comment"):
             for el in self.xpath(f"//svg:{tag}"):
                 el.getparent().remove(el)
 
@@ -891,6 +889,99 @@ class SVG:
 
         return self
 
+    def _iter_nested_svgs(
+        self, root: etree.Element
+    ) -> Generator[etree.Element, None, None]:
+        # This is different from Element.iter("svg") in that we don't yield the root
+        # svg element itself, only traverse its children and yield any immediate
+        # nested SVGs without traversing the latter's children as well.
+        frontier = deque(root)
+        while frontier:
+            el = frontier.popleft()
+            if el.tag is etree.Comment:
+                continue
+            if strip_ns(el.tag) == "svg":
+                yield el
+            elif len(el) != 0:
+                frontier.extend(el)
+
+    def _unnest_svg(
+        self, svg: etree.Element, parent_width: float, parent_height: float
+    ) -> Tuple[etree.Element, ...]:
+        x = float(svg.attrib.get("x", 0))
+        y = float(svg.attrib.get("y", 0))
+        width = float(svg.attrib.get("width", parent_width))
+        height = float(svg.attrib.get("height", parent_height))
+
+        viewport = viewbox = Rect(x, y, width, height)
+        if "viewBox" in svg.attrib:
+            viewbox = parse_view_box(svg.attrib["viewBox"])
+
+        # first recurse to un-nest any nested nested SVGs
+        self._swap_elements(
+            (el, self._unnest_svg(el, viewbox.w, viewbox.h))
+            for el in self._iter_nested_svgs(svg)
+        )
+
+        g = etree.Element(f"{{{svgns()}}}g")
+        g.extend(svg)
+
+        if viewport != viewbox:
+            preserve_aspect_ratio = svg.attrib.get("preserveAspectRatio", "xMidYMid")
+            transform = Affine2D.rect_to_rect(viewbox, viewport, preserve_aspect_ratio)
+        else:
+            transform = Affine2D.identity().translate(x, y)
+
+        if "transform" in svg.attrib:
+            transform = Affine2D.compose_ltr(
+                (transform, Affine2D.fromstring(svg.attrib["transform"]))
+            )
+
+        if transform != Affine2D.identity():
+            g.attrib["transform"] = transform.tostring()
+
+        # TODO Define a viewport-sized clipPath once transform+clip-path issue is fixed
+        # https://github.com/googlefonts/picosvg/issues/200
+        return (g,)
+
+    def resolve_nested_svgs(self, inplace=False):
+        """Replace nested <svg> elements with equivalent <g> with a transform.
+
+        NOTE: currently this is still missing two features:
+        1) resolving percentage units in reference to the nearest SVG viewport;
+        2) applying a clip to all children of the nested SVG with a rectangle the size
+           of the new viewport (inner SVGs have default overflow property set to
+           'hidden'). Blocked on https://github.com/googlefonts/picosvg/issues/200
+        No error is raised in these cases.
+
+        References:
+        - https://www.w3.org/TR/SVG/coords.html
+        - https://www.sarasoueidan.com/blog/nesting-svgs/
+        """
+        if not inplace:
+            svg = SVG(copy.deepcopy(self.svg_root))
+            svg.resolve_nested_svgs(inplace=True)
+            return svg
+
+        self._update_etree()
+
+        nested_svgs = list(self._iter_nested_svgs(self.svg_root))
+        if len(nested_svgs) == 0:
+            return
+
+        vb = self.view_box()
+        if vb is None:
+            raise ValueError(
+                "Can't determine root SVG width/height, "
+                "which is required for resolving nested SVGs"
+            )
+
+        self._swap_elements(
+            (el, self._unnest_svg(el, vb.w, vb.h)) for el in nested_svgs
+        )
+
+        return self
+
     def _select_gradients(self):
         return self.xpath(" | ".join(f"//svg:{tag}" for tag in _GRADIENT_CLASSES))
 
@@ -989,7 +1080,10 @@ class SVG:
             # only copy stops if we don't have our own
             if len(gradient) == 0:
                 for stop_el in template:
-                    gradient.append(copy.deepcopy(stop_el))
+                    new_stop_el = copy.deepcopy(stop_el)
+                    # strip stop id if present; useless and no longer unique
+                    _del_attrs(new_stop_el, "id")
+                    gradient.append(new_stop_el)
 
             del gradient.attrib[href_attr]
 
@@ -1073,6 +1167,7 @@ class SVG:
         self.remove_anonymous_symbols(inplace=True)
         self.remove_title_meta_desc(inplace=True)
         self.apply_style_attributes(inplace=True)
+        self.resolve_nested_svgs(inplace=True)
         self.shapes_to_paths(inplace=True)
         self.resolve_use(inplace=True)
         self.ungroup(inplace=True)
@@ -1098,12 +1193,8 @@ class SVG:
 
         return self
 
-    def _update_etree(self):
-        if not self.elements:
-            return
-        swaps = []
-        for old_el, shapes in self.elements:
-            swaps.append((old_el, [to_element(s) for s in shapes]))
+    @staticmethod
+    def _swap_elements(swaps: Iterable[Tuple[etree.Element, Sequence[etree.Element]]]):
         for old_el, new_els in swaps:
             for new_el in reversed(new_els):
                 old_el.addnext(new_el)
@@ -1111,6 +1202,14 @@ class SVG:
             if parent is None:
                 raise ValueError("Lost parent!")
             parent.remove(old_el)
+
+    def _update_etree(self):
+        if not self.elements:
+            return
+        self._swap_elements(
+            (old_el, [to_element(s) for s in shapes])
+            for old_el, shapes in self.elements
+        )
         self.elements = None
 
     def toetree(self):
